(* Delphi Program
   Berechnung von genherten Fermat-Zahlen
   =======================================
   siehe Simon Singh, "Homers letzter Satz"

    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   July 2022

   Ergebnis fr nmax = 10000;
    x =  280, y =  305 - i = 10 : z =   316 ->  0.07306059
    x = 1279, y = 1347 - i = 40 : z =  1351 ->  0.02719178
    x = 2323, y = 2456 - i = 65 : z =  2457 ->  0.01675662
    x = 3987, y = 4365 - i = 12 : z =  4472 ->  0.00705929    => siehe o.g. Buch
    x = 5510, y = 5822 - i = 55 : z =  5827 ->  0.00288764
    x = 7533, y = 8834 - i = 10 : z =  8999 ->  0.00283023
    x = 9112, y = 9712 - i = 76 : z =  9713 ->  0.00274480

  *)

program Fermat;

{$APPTYPE CONSOLE}

{$R *.res}

uses
  System.SysUtils, System.Math, ExtSysUtils;

const
  nmin = 51;
  nmax = 3000;
  emin = 7;
  emax = 77;
  delta = 1e-6;
var
  ix,iy,i : integer;
  x,y,z,az,d,dmin,umin: extended;
begin
  dmin:=-delta; umin:=delta;
  for ix:=nmin to nmax do begin
//      writeln('x = ',ix:4);
    for iy:=ix+1 to nmax+1 do begin
//        writeln('  y = ',iy);
      x:=ix; y:=iy;
      for i:=emin to emax do begin
        z:=IntPower(x,i)+IntPower(y,i);
        if z=MaxExtended then begin
          writeln('x = ',ix:4,', y = ',iy,' - i = ',i,' ***');
          Break;
          end;
        z:=Power(z,1/i);
        az:=Int(z+0.5);
        if az=y then Break;
        d:=z-az;          // vergleiche positive und negative Abweichungen getrennt
//        d:=abs(z-az);     // vergleiche nur absolute Abweichungen
        if (d<0) and (d>dmin) then begin
          dmin:=d;
          writeln('x = ',ix:4,', y = ',iy:4,' - i = ',i:2,' : z = ',z:5:0,' -> ',d/delta:11:8);
          end
        else if (d>=0) and (d<umin) then begin
          umin:=d;
          writeln('x = ',ix:4,', y = ',iy:4,' - i = ',i:2,' : z = ',z:5:0,' -> ',d/delta:11:8);
          end;
        if z<y+1 then Break;
        end;
      end;
    end;
  WaitForAnyKey;
end.
