(* Berechnung der Quadratwurzek aus einer Integerzahl
   siehe dazu auch: http://www.azillionmonkeys.com/qed/ulerysqroot.pdf

    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Juli 2022
   *)

program ISquareRoot;

{$APPTYPE CONSOLE}

{$R *.res}

uses
  System.SysUtils, ExtSysUtils;

type
  TLongWord = record
    case integer of
    0 : (LongWord : cardinal);
    1 : (Lo,Hi : word);
    2 : (LoL,LoH,HiL,HiH : byte);
    3 : (Bytes : array [0..3] of Byte);
    end;

  TULongWord = record
    case integer of
    0 : (ULongWord : uint64);
    1 : (Lo,Hi : cardinal);
    2 : (LoL,LoH,HiL,HiH : word);
    3 : (B0,B1,B2,B3,B4,B5,B6,B7: byte);
    4 : (Bytes : array [0..7] of Byte);
    end;

const
  XSqMask = $C0000000;
  XDSqMask = $C000000000000000;

// Integer square root
// refer to: http://www.azillionmonkeys.com/qed/ulerysqroot.pdf
function ISqrt (Value : cardinal) : cardinal;
var
  j : integer;
  tmp,sum : cardinal;
begin
  Result:=0;
  if Value>0 then begin
    tmp:=0;
    for j:=1 to 16 do begin
//      tmp:=tmp shl 2 + (Value and XSqMask) shr 30;     // shift 2 left
      tmp:=tmp shl 2 + (TLongWord(Value).HiH and $C0) shr 6;     // shift 2 left
      Value:=Value shl 2;
      Result:=Result shl 1;
      sum:=Result shl 1 +1;
      if sum<=tmp then begin
        tmp:=tmp-sum;
        Result:=Result+1;
        end;
      end;
    if 2*tmp>=sum then Result:=Result+1;  // round
    end;
  end;

function DSqrt (Value : uint64) : cardinal;
var
  j : integer;
  tmp,sum : uint64;
begin
  Result:=0;
  if Value>0 then begin
    tmp:=0;
    for j:=1 to 32 do begin
//      tmp:=tmp shl 2 + (Value and XDSqMask) shr 62;     // shift 2 left
      tmp:=tmp shl 2 + (TULongWord(Value).B7 and $C0) shr 6;     // shift 2 left
      Value:=Value shl 2;
      Result:=Result shl 1;
      sum:=Result shl 1 +1;
      if sum<=tmp then begin
        tmp:=tmp-sum;
        Result:=Result+1;
        end;
      end;
    if 2*tmp>=sum then Result:=Result+1;  // round
    end;
  end;

var
  k,n : uint64;
begin
  k:=10000000000000000;
//  n:=isqrt(k);
  n:=dsqrt(k);
  writeln(Format('Sqrt(%u) = %u',[k,n]));

  WaitForAnyKey;
  end.
